@@ -5,7 +5,8 @@
  *                    Brendan Cully <brendan@xiph.org>,
  *                    Karl Heyes <karl@xiph.org>,
  *                    Jack Moffitt <jack@icecast.org>,
- *                    Ed "oddsock" Zaleski <oddsock@xiph.org>
+ *                    Ed "oddsock" Zaleski <oddsock@xiph.org>,
+ * Copyright (C) 2014-2019 Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -51,10 +52,16 @@
 #include <arpa/inet.h>
 #include <sys/time.h>
 #include <netdb.h>
-#else
+#endif
+
+#ifdef HAVE_WINSOCK2_H
 #include <winsock2.h>
 #endif
 
+#ifdef HAVE_WS2TCPIP_H
+#include <ws2tcpip.h>
+#endif
+
 #include "sock.h"
 #include "resolver.h"
 
@@ -95,26 +102,6 @@ void sock_shutdown(void)
     resolver_shutdown();
 }
 
-/* sock_get_localip
-**
-** gets the local ip address for the machine
-** the ip it returns *should* be on the internet.
-** in any case, it's as close as we can hope to get
-** unless someone has better ideas on how to do this
-*/
-char *sock_get_localip(char *buff, int len)
-{
-    char temp[1024];
-
-    if (gethostname(temp, sizeof(temp)) != 0)
-        return NULL;
-
-    if (resolver_getip(temp, buff, len))
-        return buff;
-
-    return NULL;
-}
-
 /* sock_error
 ** 
 ** returns the last socket error
@@ -168,32 +155,6 @@ int sock_recoverable(int error)
     }
 }
 
-int sock_stalled (int error)
-{
-    switch (error)
-    {
-    case EAGAIN:
-    case EINPROGRESS:
-    case EALREADY:
-#if defined(EWOULDBLOCK) && EWOULDBLOCK != EAGAIN
-    case EWOULDBLOCK:
-#endif
-#if defined (WSAEWOULDBLOCK) && WSAEWOULDBLOCK != EWOULDBLOCK
-    case WSAEWOULDBLOCK:
-#endif
-#if defined (WSAEINPROGRESS) && WSAEINPROGRESS != EINPROGRESS
-    case WSAEINPROGRESS:
-#endif
-#ifdef ERESTART
-    case ERESTART:
-#endif
-        return 1;
-    default:
-        return 0;
-    }
-}
-
-
 static int sock_connect_pending (int error)
 {
     return error == EINPROGRESS || error == EALREADY;
@@ -312,45 +273,6 @@ int sock_close(sock_t sock)
 #endif
 }
 
-/* sock_writev
- *
- * write multiple buffers at once, return bytes actually written
- */
-#ifdef HAVE_WRITEV
-
-ssize_t sock_writev (sock_t sock, const struct iovec *iov, size_t count)
-{
-    return writev (sock, iov, count);
-}
-
-#else
-
-ssize_t sock_writev (sock_t sock, const struct iovec *iov, size_t count)
-{
-    int i = count, accum = 0, ret;
-    const struct iovec *v = iov;
-
-    while (i)
-    {
-        if (v->iov_base && v->iov_len)
-        {
-            ret = sock_write_bytes (sock, v->iov_base, v->iov_len);
-            if (ret == -1 && accum==0)
-                return -1;
-            if (ret == -1)
-                ret = 0;
-            accum += ret;
-            if (ret < (int)v->iov_len)
-                break;
-        }
-        v++;
-        i--;
-    }
-    return accum;
-}
-
-#endif
-
 /* sock_write_bytes
 **
 ** write bytes to the socket
@@ -370,16 +292,6 @@ int sock_write_bytes(sock_t sock, const void *buff, size_t len)
     return send(sock, buff, len, 0);
 }
 
-/* sock_write_string
-**
-** writes a string to a socket
-** This function must only be called with a blocking socket.
-*/
-int sock_write_string(sock_t sock, const char *buff)
-{
-    return (sock_write_bytes(sock, buff, strlen(buff)) > 0);
-}
-
 /* sock_write
 **
 ** write a formatted string to the socket
@@ -711,51 +623,66 @@ sock_t sock_connect_wto_bind (const char *hostname, int port, const char *bnd, i
 }
 
 
-sock_t sock_get_server_socket (int port, const char *sinterface)
+sock_t sock_get_server_socket (int port, const char *sinterface, bool prefer_inet6)
 {
     struct sockaddr_storage sa;
     struct addrinfo hints, *res, *ai;
-    char service [10];
+    char service [16];
     int sock;
 
     if (port < 0)
         return SOCK_ERROR;
 
-    memset (&sa, 0, sizeof(sa));
-    memset (&hints, 0, sizeof(hints));
-
-    hints.ai_family = AF_UNSPEC;
-    hints.ai_flags = AI_PASSIVE | AI_ADDRCONFIG | AI_NUMERICSERV | AI_NUMERICHOST;
-    hints.ai_socktype = SOCK_STREAM;
-    snprintf (service, sizeof (service), "%d", port);
-
-    if (getaddrinfo (sinterface, service, &hints, &res))
-        return SOCK_ERROR;
-    ai = res;
-    do
-    {
-        int on = 1;
-        sock = socket (ai->ai_family, ai->ai_socktype, ai->ai_protocol);
-        if (sock < 0)
-            continue;
-
-        setsockopt (sock, SOL_SOCKET, SO_REUSEADDR, (const void *)&on, sizeof(on));
-        on = 0;
-#ifdef IPV6_V6ONLY
-        setsockopt (sock, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof on);
+#ifndef AF_INET6
+    prefer_inet6 = false;
 #endif
 
-        if (bind (sock, ai->ai_addr, ai->ai_addrlen) < 0)
-        {
-            sock_close (sock);
-            continue;
+    memset(&sa,    0, sizeof(sa));
+    memset(&hints, 0, sizeof(hints));
+
+    hints.ai_family   = AF_UNSPEC;
+    hints.ai_flags    = AI_PASSIVE | AI_ADDRCONFIG | AI_NUMERICSERV | AI_NUMERICHOST;
+    hints.ai_socktype = SOCK_STREAM;
+    snprintf(service, sizeof(service), "%d", port);
+
+    if (getaddrinfo(sinterface, service, &hints, &res))
+        return SOCK_ERROR;
+
+    while (true) {
+        for (ai = res; ai; ai = ai->ai_next) {
+            int on = 1;
+
+#ifdef AF_INET6
+            if (prefer_inet6 && ai->ai_family != AF_INET6)
+                continue;
+#endif
+
+            sock = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
+            if (sock < 0)
+                continue;
+
+            setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (const void *)&on, sizeof(on));
+
+#ifdef IPV6_V6ONLY
+            on = 0;
+            setsockopt(sock, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof(on));
+#endif
+
+            if (bind(sock, ai->ai_addr, ai->ai_addrlen) < 0){
+                sock_close(sock);
+                continue;
+            }
+
+            freeaddrinfo(res);
+            return sock;
         }
-        freeaddrinfo (res);
-        return sock;
 
-    } while ((ai = ai->ai_next));
+        if (!prefer_inet6)
+            break;
+        prefer_inet6 = false;
+    }
 
-    freeaddrinfo (res);
+    freeaddrinfo(res);
     return SOCK_ERROR;
 }
 
@@ -863,13 +790,16 @@ sock_t sock_connect_wto_bind (const char *hostname, int port, const char *bnd, i
 ** interface.  if interface is null, listen on all interfaces.
 ** returns the socket, or SOCK_ERROR on failure
 */
-sock_t sock_get_server_socket(int port, const char *sinterface)
+sock_t sock_get_server_socket (int port, const char *sinterface, bool prefer_inet6)
 {
     struct sockaddr_in sa;
     int error, opt;
     sock_t sock;
     char ip[MAX_ADDR_LEN];
 
+    /* Not supported. */
+    (void)prefer_inet6;
+
     if (port < 0)
         return SOCK_ERROR;
 
@@ -960,3 +890,102 @@ sock_t sock_accept(sock_t serversock, char *ip, size_t len)
     return ret;
 }
 
+bool sock_is_ipv4_mapped_supported(void)
+{
+#ifdef AF_INET6
+    struct sockaddr_storage ssock_addr;
+    struct sockaddr_in6 *ssock_addr_in6 = (struct sockaddr_in6 *)&ssock_addr;
+    socklen_t ssock_addr_len = sizeof(ssock_addr);
+    sock_t ssock = SOCK_ERROR;
+    sock_t csock;
+    char ip[MAX_ADDR_LEN+1];
+
+    ssock = sock_get_server_socket(0, "::", true);
+
+    if (ssock == SOCK_ERROR)
+        return false;
+
+    if (listen(ssock, 1) != 0) {
+        sock_close(ssock);
+        return false;
+    }
+
+    if (getsockname(ssock, (struct sockaddr*)&ssock_addr, &ssock_addr_len) != 0) {
+        sock_close(ssock);
+        return false;
+    }
+
+    if (ssock_addr_len < sizeof(struct sockaddr_in6) || ssock_addr_in6->sin6_family != AF_INET6) {
+        sock_close(ssock);
+        return false;
+    }
+
+    csock = sock_connect("127.0.0.1", ntohs(ssock_addr_in6->sin6_port));
+    if (csock == SOCK_ERROR) {
+        sock_close(ssock);
+        return false;
+    } else {
+        sock_close(csock);
+    }
+
+    csock = sock_accept(ssock, ip, sizeof(ip));
+    if (csock == SOCK_ERROR) {
+        sock_close(ssock);
+        return false;
+    } else {
+        sock_close(csock);
+    }
+
+    sock_close(ssock);
+
+    return strcmp(ip, "::ffff:127.0.0.1") == 0 ? true : false;
+#else
+    return false;
+#endif
+}
+
+sock_family_t sock_get_family(sock_t sock)
+{
+    struct sockaddr addr;
+    socklen_t len = sizeof(addr);
+
+    if (getsockname(sock, &addr, &len) != 0)
+        return SOCK_FAMILY__ERROR;
+
+    if (len < sizeof(addr.sa_family))
+        return SOCK_FAMILY__ERROR;
+
+    switch (addr.sa_family) {
+#ifdef AF_UNSPEC
+        case AF_UNSPEC: return SOCK_FAMILY_UNSPEC; break;
+#endif
+#ifdef AF_UNIX
+        case AF_UNIX: return SOCK_FAMILY_UNIX; break;
+#endif
+#ifdef AF_INET
+        case AF_INET: return SOCK_FAMILY_INET4; break;
+#endif
+#ifdef AF_INET6
+        case AF_INET6: return SOCK_FAMILY_INET6; break;
+#endif
+#ifdef AF_DECnet
+        case AF_DECnet: return SOCK_FAMILY_DECnet; break;
+#endif
+    }
+
+    return SOCK_FAMILY__ERROR;
+}
+
+const char * sock_family_to_string(sock_family_t family)
+{
+    switch (family) {
+        case SOCK_FAMILY__ERROR: return "<error>"; break;
+        case SOCK_FAMILY_UNSPEC: return "UNSPEC"; break;
+        case SOCK_FAMILY_UNIX: return "UNIX"; break;
+        case SOCK_FAMILY_INET4: return "INET4"; break;
+        case SOCK_FAMILY_INET6: return "INET6"; break;
+        case SOCK_FAMILY_DECnet: return "DECnet"; break;
+    }
+
+    return "<unknown>";
+}
