@@ -187,7 +187,9 @@ static shout_connection_return_state_t shout_connection_iter__socket(shout_conne
         break;
         case SHOUT_SOCKSTATE_TLS_CONNECTING:
         case SHOUT_SOCKSTATE_TLS_CONNECTED:
-            rc = shout_tls_try_connect(con->tls);
+            do {
+                rc = shout_tls_try_connect(con->tls);
+            } while (rc == SHOUTERR_RETRY);
             if (rc == SHOUTERR_SUCCESS) {
                 con->current_socket_state = SHOUT_SOCKSTATE_TLS_VERIFIED;
                 return SHOUT_RS_DONE;
@@ -563,7 +565,7 @@ int                 shout_connection_connect(shout_connection_t *con, shout_t *s
     if (con->socket != SOCK_ERROR || con->current_socket_state != SHOUT_SOCKSTATE_UNCONNECTED)
         return SHOUTERR_BUSY;
 
-    if (con->nonblocking == SHOUT_BLOCKING_DEFAULT)
+    if (con->nonblocking != SHOUT_BLOCKING_DEFAULT)
         shout_connection_set_nonblocking(con, shout_get_nonblocking(shout));
 
     port = shout->port;
